/*
 * $Id$
 *
 * XS interface to the Win32 Common Dialogs
 * based on Registry.CPP written by Jesse Dougherty
 *
 * $Log$
 *
 */

#define  WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <commdlg.h>
#include <cderr.h>
#include <string.h>

#include "EXTERN.h"
#include "perl.h"
#include "XSUB.h"
// Section for the constant definitions.
#define CROAK croak
#define CARP  warn
#define MAX_LENGTH 2048
#define TMPBUFSZ 1024
#define MY_PACKAGE "Win32::CommDlg"
#define MY_USAGE "usage: " MY_PACKAGE "::"

typedef unsigned short word;
typedef unsigned int UINT;

// VALUES FILLED IN BY PerlCallback
// DWORD status=-1;
// static CPerl* myPerl;

#define SUCCESSRETURNED(x)	(x == ERROR_SUCCESS)
#define INETRETURN(x) XSRETURN_IV(SUCCESSRETURNED(x))

#define SETIV(index,value) sv_setiv(ST(index), value)
#define SETPV(index,string) sv_setpv(ST(index), string)
#define SETPVN(index, buffer, length) sv_setpvn(ST(index), (char*)buffer, length)
#define SETHKEY(index, hkey)	SETIV(index,(long)hkey)

#define NEW(x,v,n,t)  (v = (t*)safemalloc((MEM_SIZE)((n) * sizeof(t))))
#define PERLSvIV(sv) (SvIOK(sv) ? SvIVX(sv) : sv_2iv(sv))
#define PERLSvPV(sv, lp) (SvPOK(sv) ? ((lp = SvCUR(sv)), SvPVX(sv)) : sv_2pv(sv, &lp))

#define PERLPUSHMARK(p) if (++markstack_ptr == markstack_max)	\
			markstack_grow();			\
		    *markstack_ptr = (p) - stack_base

#define PERLXPUSHs(s)	do {\
 		if (stack_max - sp < 1) {\
			    sp = stack_grow(sp, sp, 1);\
			}\
  (*++sp = (s)); } while (0)


#ifdef NDEBUG
#define DEBUG_WARN warn
#else
#define DEBUG_WARN
#endif

#include "constant.cpp"

XS(XS_Win32__CommDlg_constant)
{
    dXSARGS;

    if (items != 2) {
		croak("Usage: Win32::CommDlg::constant(name,arg)");
    }
    {
		char *	name = (char *)SvPV(ST(0),na);
		int	arg = (int)SvIV(ST(1));
		DWORD RETVAL;

		RETVAL = constant(pPerl,name, arg);
		ST(0) = sv_newmortal();
		sv_setiv(ST(0), (IV)RETVAL);
    }
    XSRETURN(1);
}


//-----------------------------------------------------------------
XS(XS_Win32__CommDlg_MessageBox)
{
	dXSARGS;
    char *caption, *text;
    UINT type, res;

	if(1>items || items>3)
	{
		CROAK(MY_USAGE "MessageBox($text[, $caption, $flags]);\n");
	}
    text=PERLSvPV(ST(0), na);

    if(items >= 2)
        caption=PERLSvPV(ST(1), na);
    else
        caption="Perl";

    if(items>=3)
        type=PERLSvIV(ST(2));
    else
        type=MB_OK|MB_TASKMODAL;

    res=MessageBoxEx(NULL, text, caption, type,
        ((unsigned int)(LANG_NEUTRAL))|((unsigned int)(SUBLANG_DEFAULT)<<10));
    // res=MessageBox(NULL, text, caption, type);
    if(res==0)
    {
        XSRETURN_NO;
    }
    EXTEND(sp, 1);
    ST(0)=sv_2mortal(newSViv(res));
    XSRETURN(1);
}

//-----------------------------------------------------------------
XS(XS_Win32__CommDlg_CommDlgExtendedError)
{
	dXSARGS;
    ST(0)=sv_2mortal(newSViv(CommDlgExtendedError()));
    XSRETURN(1);
}

//-----------------------------------------------------------------
static char szFilter[] = "All Files (*.*)\0*.*\0\0" \
                         "Text Files (*.TXT)\0*.txt\0" ;

#define BUFFER_SIZE (5*1024)
void InitFN(OPENFILENAME *ofn, char *dir, char *caption, UINT flags, char *file, char *filters, CPerl *pPerl)
{
    char *buffer;
    if(dir!=NULL && dir[0]==0)
        dir=NULL;
    if(file!=NULL && file[0]==0)
        file=NULL;
    if(caption!=NULL && caption[0]==0)
        caption=NULL;
    if(filters==NULL)
        filters=szFilter;
    if(flags==0)
        flags=OFN_HIDEREADONLY;

    Newz(0, buffer, BUFFER_SIZE, char);
    if(buffer==NULL)
        CROAK(MY_PACKAGE " Out of memory!");

    if(file!=NULL)
        strcpy(buffer, file);

    ofn->lStructSize       = sizeof (OPENFILENAME) ;
    ofn->hwndOwner         = GetActiveWindow() ;
/*    if(ofn->hwndOwner==NULL)
        ofn->hwndOwner         = GetTopWindow(NULL) ;
*/
    ofn->hInstance         = NULL ;
    ofn->lpstrFilter       = filters;
    ofn->lpstrCustomFilter = NULL ;
    ofn->nMaxCustFilter    = 0 ;
    ofn->nFilterIndex      = 1 ;
    ofn->lpstrFile         = buffer ;
    ofn->nMaxFile          = (BUFFER_SIZE*sizeof(char));
    ofn->lpstrFileTitle    = NULL ;
    ofn->nMaxFileTitle     = 0 ;
    ofn->lpstrInitialDir   = dir ;
    ofn->lpstrTitle        = caption ;
    ofn->Flags             = flags|OFN_EXPLORER;
    ofn->nFileOffset       = 0 ;
    ofn->nFileExtension    = 0 ;
    ofn->lpstrDefExt       = NULL ;
    ofn->lCustData         = 0L ;
    ofn->lpfnHook          = NULL ;
    ofn->lpTemplateName    = NULL ;
}

XS(XS_Win32__CommDlg__GetFileName)
{
	dXSARGS;
    OPENFILENAME fn;
    UINT res, flags=0, Ok=0, save=0;
    char *dir=NULL, *caption=NULL;
    AV *files;
    SV *sv;

    if(SvIOK(ST(0)))
        Ok = 1;
	sv = ST(0) ;
	if(SvROK(sv))
		sv = SvRV(sv) ;
	Ok = Ok && (SvTYPE(sv) == SVt_PVAV) ;
	files = (AV*)sv ;

	if(Ok && (2>items || items>4)) {
		CROAK(MY_USAGE "_GetOpenFileName(\\@files, $save, [$dir, $caption, $flags]);\n");
	}

    av_clear(files);
    save=PERLSvIV(ST(1));

    if(items>=3)
        dir=PERLSvPV(ST(2), na);
    if(items>=4)
        caption=PERLSvPV(ST(3), na);
    if(items>=5)
        flags=PERLSvIV(ST(4), na);

    InitFN(&fn, dir, caption, flags, NULL, NULL, pPerl);
    if(save)
        res=GetSaveFileName(&fn);
    else
        res=GetOpenFileName(&fn);

    if(res==0)
    {
        Safefree(fn.lpstrFile);
        XSRETURN_NO;
    }

    if(fn.Flags | OFN_ALLOWMULTISELECT)
    {
        char *name;
        int len=0, q=0;

        for(name=fn.lpstrFile; name[0]!=0; name+=len+1)
        {
            DEBUG_WARN("--- %s\n", name);
            len=strlen(name);
			av_push(files,  newSVpv(name, len));
            q++;
        }
    } else
    {
        DEBUG_WARN("-- %s\n", fn.lpstrFile);
        av_push(files, newSVpv(fn.lpstrFile, strlen(fn.lpstrFile)));
    }
//    Safefree(fn.lpstrFile);
    XSRETURN_YES;
}

//-----------------------------------------------------------------
XS(boot_Win32__CommDlg)
{
	dXSARGS;
	char* file = __FILE__;

	newXS("Win32::CommDlg::constant", XS_Win32__CommDlg_constant, file);

	newXS("Win32::CommDlg::MessageBox", XS_Win32__CommDlg_MessageBox, file);
	newXS("Win32::CommDlg::_GetFileName", XS_Win32__CommDlg__GetFileName, file);
	newXS("Win32::CommDlg::CommDlgExtendedError", XS_Win32__CommDlg_CommDlgExtendedError, file);

	ST(0) = &sv_yes;
	XSRETURN(1);
}

