package Win32::CommDlg;
#######################################################################
#
# $Id$
#
# $Log$
#######################################################################

require Exporter;       # to export the constants to the main:: space
require DynaLoader;     # to dynaload the module.

use Win32::WinError;    # to windows constants.
use Carp;               # to complain with style
use strict;

@Win32::CommDlg::ISA=qw( Exporter DynaLoader );
@Win32::CommDlg::RetExp=qw(
                IDABORT
                IDCANCEL
                IDIGNORE
                IDNO
                IDOK
                IDRETRY
                IDYES);
@Win32::CommDlg::MBExp=qw(
                MB_APPLMODAL
                MB_ABORTRETRYIGNORE
                MB_DEFBUTTON1
                MB_DEFBUTTON2
                MB_DEFBUTTON3
                MB_DEFBUTTON4
                MB_HELP
                MB_ICONASTERISK
                MB_ICONERROR
                MB_ICONEXCLAMATION
                MB_ICONHAND
                MB_ICONINFORMATION
                MB_ICONQUESTION
                MB_ICONSTOP
                MB_ICONWARNING
                MB_OK
                MB_OKCANCEL
                MB_RETRYCANCEL
                MB_RIGHT
                MB_RTLREADING
                MB_SERVICE_NOTIFICATION
                MB_SETFOREGROUND
                MB_SYSTEMMODAL
                MB_TASKMODAL
                MB_TOPMOST
                MB_YESNO
                MB_YESNOCANCEL
            );
@Win32::CommDlg::OFNExp=qw(
                OFN_ALLOWMULTISELECT
                OFN_CREATEPROMPT
                OFN_FILEMUSTEXIST
                OFN_HIDEREADONLY
                OFN_NOCHANGEDIR
                OFN_NODEREFERENCELINKS
                OFN_NONETWORKBUTTON
                OFN_NOREADONLYRETURN
                OFN_NOTESTFILECREATE
                OFN_NOVALIDATE
                OFN_OVERWRITEPROMPT
                OFN_PATHMUSTEXIST
                OFN_READONLY
                OFN_SHAREAWARE
            );
@Win32::CommDlg::ErrorExp=qw(
                CDERR_DIALOGFAILURE
                CDERR_FINDRESFAILURE
                CDERR_INITIALIZATION
                CDERR_LOADRESFAILURE
                CDERR_LOADSTRFAILURE
                CDERR_LOCKRESFAILURE
                CDERR_MEMALLOCFAILURE
                CDERR_MEMLOCKFAILURE
                CDERR_NOHINSTANCE
                CDERR_NOHOOK
                CDERR_NOTEMPLATE
                CDERR_REGISTERMSGFAIL
                CDERR_STRUCTSIZE
                CFERR_MAXLESSTHANMIN
                CFERR_NOFONTS
                FNERR_BUFFERTOOSMALL
                FNERR_INVALIDFILENAME
                FNERR_SUBCLASSFAILURE
                FRERR_BUFFERLENGTHZERO
                PDERR_CREATEICFAILURE
                PDERR_DEFAULTDIFFERENT
                PDERR_DNDMMISMATCH
                PDERR_GETDEVMODEFAIL
                PDERR_INITFAILURE
                PDERR_LOADDRVFAILURE
                PDERR_NODEFAULTPRN
                PDERR_NODEVICES
                PDERR_PARSEFAILURE
                PDERR_PRINTERNOTFOUND
                PDERR_RETDEFFAILURE
                PDERR_SETUPFAILURE
            );

@Win32::CommDlg::InternalExp=qw(
                _GetOpenFileName
                _GetSaveFileName
            );

@Win32::CommDlg::FuncExp=qw(
                Alert
                Ask
                Confirm
                MessageBox
                GetFileName
                GetMultiFileName
                GetOpenFileName
                GetSaveFileName
                CommDlgExtendedError
                $ID
                $MB
                $OFN
            );

%Win32::CommDlg::EXPORT_TAGS=(
            RETURN=>[@Win32::CommDlg::RetExp],
            MB_FLAGS=>[@Win32::CommDlg::MBExp],
            OFN_FLAGS=>[@Win32::CommDlg::OFNExp],
            ERROR=>[@Win32::CommDlg::ErrorExp],
            FUNCTIONS=>[@Win32::CommDlg::FuncExp],
        );


@Win32::CommDlg::EXPORT_OK=(@Win32::CommDlg::FuncExp,
                            @Win32::CommDlg::MBExp,
                            @Win32::CommDlg::OFNExp,
                            @Win32::CommDlg::RetExp,
                            @Win32::CommDlg::ErrorExp,
                            @Win32::CommDlg::InternalExp,
                            );

#######################################################################
#### PACKAGE GLOBALS
$Win32::CommDlg::VERSION='0.01';
$Win32::CommDlg::PACKAGE='Win32::CommDlg';
$Win32::CommDlg::ID=new Win32::CommDlg::ID;
$Win32::CommDlg::MB=new Win32::CommDlg::MB;
$Win32::CommDlg::OFN=new Win32::CommDlg::OFN;

#######################################################################
#### PROTOTYPES
sub MessageBox ($;$$);
sub CommDlgExtendedError ();
# sub _GetFileName ($$;$$$);
sub MB_OK {0}       # zero won't get past AUTOLOAD

################################################################
#### NOT FOR EXTERNAL CONSUMPTION
# handles error conditions for _GetFileName
# (\\@files, $save, [$dir, $caption, $flags])
sub _GetFile
{
    if(_GetFileName(@_))
    {
        return 1;
    } else
    {
        if(CommDlgExtendedError()==&FNERR_BUFFERTOOSMALL)
        {
            carp "$Win32::CommDlg::PACKAGE : internal error, hard-coded value is to small.\n"
        }
    }
    return 0;
}


###############################################
# bare-bones wrapers
# arrayref dir file flags
sub _GetOpenFileName ($;$$$)
{
    unshift @_, shift(@_), 0;
    return _GetFileName(@_);
}

# arrayref dir file flags
sub _GetSaveFileName ($;$$$)
{
    unshift @_, shift(@_), 1;
    return _GetFileName(@_);
}


#######################################################################
#### PUBLIC METHODS
#
sub Version { $Win32::CommDlg::VERSION }


###############################################
# msg caption
sub Alert ($;$)
{
    MessageBox($_[0], (defined($_[1])?$_[1] : 'Perl Alert'), &MB_OK|&MB_ICONWARNING|&MB_TASKMODAL);
}

###############################################
# msg caption
sub Confirm ($;$)
{
    return (MessageBox($_[0], (defined($_[1])?$_[1] : 'Perl Confirm'), &MB_OKCANCEL|&MB_ICONQUESTION|&MB_TASKMODAL)
            ==&IDOK);
}
###############################################
# msg caption
sub Ask ($;$)
{
    return (MessageBox($_[0], (defined($_[1])?$_[1] : 'Perl Question'), &MB_YESNO|&MB_ICONQUESTION|&MB_TASKMODAL)
            ==&IDYES);
}



#######################################################################
# dir file flags
sub GetSaveFileName (;$$$)
{
    my @ret=();
    _GetFile(\@ret, 1, @_);
    return $ret[0];
}
#######################################################################
# dir file flags
sub GetOpenFileName (;$$$)
{
    my @ret=();
    _GetFile(\@ret, 0, @_);
    return $ret[0];
}
#######################################################################
# dir file
sub GetFileName (;$$)
{
    return GetOpenFileName(@_);
}
#######################################################################
# dir file flags
sub GetMultiFileName (;$$$)
{
    my @ret;
    if(_GetFile(\@ret, 0,
                        (defined($_[0])?$_[0]:''),
                        (defined($_[1])?$_[1]:''),
                        (defined($_[2])?$_[2]:&OFN_FILEMUSTEXIST|&OFN_HIDEREADONLY)|&OFN_ALLOWMULTISELECT,
                      ))
    {
        return $ret[0] unless(scalar(@ret)>1);
        my $path=shift @ret;
        return map {$path.'\\'.$_} @ret;
    }
    return ();
}

#######################################################################
#### dynamically load in the CommDlg.pll module.
#

bootstrap Win32::CommDlg;

# Preloaded methods go here.

#######################################################################
### AUTOLOAD
# This AUTOLOAD is used to 'autoload' constants from the constant()
# XS function.  If a constant is not found then control is passed
# to the AUTOLOAD in AutoLoader.
sub AUTOLOAD {
    my($constname);
    ($constname = $Win32::CommDlg::AUTOLOAD) =~ s/.*:://;
    #reset $! to zero to reset any current errors.
    $!=0;

#    my $val = constant($constname, @_ ? $_[0] : 0);
    my $val = constant($constname, 0);

    if ($! != 0)
    {

        # [dada] This results in an ugly Autoloader error
        #if ($! =~ /Invalid/) {
        #    $AutoLoader::AUTOLOAD = $AUTOLOAD;
        #    goto &AutoLoader::AUTOLOAD;
        #} else {
        # [dada] ... I prefer this one :)

            my($pack,$file,$line) = caller;
            die "Win32::CommDlg::$constname is not defined, used at $file line $line.\n";
        #}
    }
    printf STDERR "- $constname = 0x%X\n", $val;
    eval "sub $Win32::CommDlg::AUTOLOAD { $val }";
    goto &$Win32::CommDlg::AUTOLOAD;
}

#######################################################################
#######################################################################
package Win32::CommDlg::ID;
use strict;

sub AUTOLOAD
{
    my @constant=split('::', $Win32::CommDlg::ID::AUTOLOAD);
    $constant[-1]='ID'.$constant[-1] unless($constant[-1]=~/^ID/);
    $Win32::CommDlg::AUTOLOAD=join '::', @constant;
    my $jump='Win32::CommDlg::AUTOLOAD';
    goto &$jump;
}
sub new { return bless {}, shift; }

#######################################################################
package Win32::CommDlg::MB;
use strict;

sub AUTOLOAD
{
    my @constant=split('::', $Win32::CommDlg::MB::AUTOLOAD);
    $constant[-1]='MB_'.$constant[-1] unless($constant[-1]=~/^MB_/);
    $Win32::CommDlg::AUTOLOAD=join '::', @constant;
    my $jump='Win32::CommDlg::AUTOLOAD';
    goto &$jump;
}
sub new { return bless {}, shift; }

#######################################################################
package Win32::CommDlg::OFN;
use strict;

sub AUTOLOAD
{
    my @constant=split('::', $Win32::CommDlg::OFN::AUTOLOAD);
    $constant[-1]='OFN_'.$constant[-1] unless($constant[-1]=~/^OFN_/);
    $Win32::CommDlg::AUTOLOAD=join '::', @constant;
    my $jump='Win32::CommDlg::AUTOLOAD';
    goto &$jump;
}
sub new { return bless {}, shift; }


1;
__END__


#######################################################################

=head1 NAME

Win32::CommDLg - a Perl interface to the commdlg.dll

=head1 DESCRIPTION

Hey, I've been meaning to do this for a while, and I think it's fun.  I also
hope you'll find it useful.

=head1 CONTENTS

L<"FUNCTIONS"> -
L<"CONSTANTS"> -
L<"AUTHOR"> -
L<"BUGS">

=head2 FUNCTIONS

L<Alert> -
L<Ask> -
L<Confirm> -
L<GetFileName> -
L<GetMultiFileName>

=over 4

=item Alert

    Alert($msg, [$caption]);

Pops a message box on to the screen, with the ! icon and that annoying
sound. $msg is what is what you want to say and $caption appears in the
head of the message box.  $caption defaults to 'Perl Alert'.

Example:

    open("<$file") or Alert("Unable to open $file : $!") and die "\n";

=item Confirm

    Confirm($msg, [$caption]);

Pops a message box on to the screen, with the ? icon and an annoying
beep. $msg is what is what you want to know and $caption appears in the
head of the message box.  $caption defaults to 'Perl Confirm'.

The user may chose between a I<OK> or I<Cancel> buttons.  Returns true
if the user presses I<OK>, false otherwise.

Example:

    if(Confirm("Do you really want me to delete $file?"))
    {
        unlink($file);
    }

=item Ask

    Ask($msg, [$caption]);

Pops a message box on to the screen, with the ? icon and an annoying
noise. $msg is what is what you want to know and $caption appears in the
head of the message box.  $caption defaults to 'Perl Question'.

The user may chose between a I<Yes> or I<No> buttons.  Returns true
if the user presses I<Yes>, false otherwise.

Example:

    $fries=Ask("Do you want fries with that?");

=item GetFileName

    GetFileName([$dir, $caption])

Prompt the user for a file name those cool Explorer style dialog boxes.
$dir is the directory to start looking in (default is cwd) and $caption
is what goes in the title (default is 'Open file').
Returns the full pathname to the file, or nada if the user pressed I<Cancel>.

Example:

    $file=GetFileName('', 'Where are the crown jewels?');
    if($file)
    {
        Confirm("Are you sure there here?\n$file");
    }

=item GetMultiFilename

    GetMultiFilename([$dir, $caption])

Prompt the user for several file names those cool Explorer style dialog boxes.
$dir is the directory to start looking in (default is cwd) and $caption
is what goes in the title (default is 'Open file').
Returns an array of full pathnames to the files,
or an empty array if the user pressed I<Cancel>.

Example:

    @files=GetMultiFilename('c:\', 'Files to delete');
    print "Bwahahaha!\n"
    foreach $file (@files)
    {
        unlink $file;
    }

=back

=head2 CONSTANTS

    $ID, $MB, $OFN

COMMDLG.H has more constants than you can shake a stick at.  Rather than
exporting them into your name space, I set up so little objects that
magically AUTOLOAD from the Win32::CommDlg namespace.  $ID->YES
will return the C<IDYES> constant,  $MB->ICONEXCLAIM is
C<MB_ICONEXCLAIM>, and so on. These objects are smart enough that they
won't choke on $ID->IDCANCEL, if you type that my accident.

You may also import the constants with the C<:ID_FLAGS>, C<:MB_FLAGS> and
C<:OFN_FLAGS>
import tags.  In which case the constants are accessed via C<&MB_YESNO>,
which is better than C<&Win32::CommDlg::MB_YESNO>, but only just.

BTW, this is an idea I thought up all by my lonesome.  And I'm somewhat
proud of it C<-:)>

Example:

    if(MessageBox("Here's something very interesing.", $0,
                    $MB->YESNO|$MB->ICONEXCLAMATION|$MB->TASKMODAL)==$ID->YES)
    {
        print <<HERE;
    \$MB->YESNO=@{[$MB->YESNO]}
    \$MB->ICONWARNING=@{[$MB->ICONWARNING]}
    \$ID->YES=@{[$ID->YES]}
    \$ID->ABORT=@{[$ID->ABORT]}
    \$ID->CANCEL=@{[$ID->CANCEL]}
    HERE
    }



